/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: vtc.h
 * Date: 02.12.2013
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>, Zdenek Pohl <xpohl@utia.cas.cz> *
 * Description:
 *    Implements functions for handling Video Timing Controller
 *
 * Changelog:
 * aarch64  compatibility
 *
 *
 *
 ********************************************/

#ifndef _VTC_HEADER_FILE_
#define _VTC_HEADER_FILE_

// #include <xvtc.h>
#include "xvtc.h"

#define XVTC_ERR_A_CHR_LOCK_MASK    0x00200000
#define XVTC_ERR_A_VID_LOCK_MASK    0x00100000
#define XVTC_ERR_HSYNC_LOCK_MASK    0x00080000
#define XVTC_ERR_VSYNC_LOCK_MASK    0x00040000
#define XVTC_ERR_HBLANK_LOCK_MASK   0x00020000
#define XVTC_ERR_VBLANK_LOCK_MASK   0x00010000
#define XVTX_ERR_ALL_MASK           ( XVTC_ERR_A_CHR_LOCK_MASK  | \
                                      XVTC_ERR_A_VID_LOCK_MASK  | \
                                      XVTC_ERR_HSYNC_LOCK_MASK  | \
                                      XVTC_ERR_VSYNC_LOCK_MASK  | \
                                      XVTC_ERR_HBLANK_LOCK_MASK | \
                                      XVTC_ERR_VBLANK_LOCK_MASK ) \


#define VTC_SEL_NONE      255 // not selected
#define VTC_1920x1080p60  0
#define VTC_1920x1200p50  1
#define VTC_1600x1200p50  2
#define VTC_1680x1050p60  3
#define VTC_1280x1024p60  4
#define VTC_1280x720p60   5
#define VTC_1024x768p60   6
#define VTC_800x600p60    7
#define VTC_640x480p60    8
#define VTC_600x800p60    9


#define VTC_RES_CNT     10

struct struct_vtc_timing_t
{
    char *name;

    u32   vav;    // vertical active video
    u32   vfp;    // vertical front porch
    u32   vsw;    // vertical sync width
    u32   vbp;    // vertical back porch
    u32   vsp;    // vertical sync polarity
    u32   vblp;   // vertical blank polarity

    u32   hav;    // horizontal active video
    u32   hfp;    // horizontal front porch
    u32   hsw;    // horizontal sync width
    u32   hbp;    // horizontal back porch
    u32   hsp;    // horizontal sync polarity
    u32   hblp;   // horizontal blank polarity

    float pclk;
};
typedef struct struct_vtc_timing_t vtc_timing_t;

struct struct_vtc_t
{
    XVtc              inst;
    XVtc_Signal       signal;       // parameters of video signal
    XVtc_Polarity     polarity;     // used polarity
    XVtc_SourceSelect src_sel;      // source selection setting
    u32               locked;
    u8                timing_sel;   // 0 - 1920x1080
};
typedef struct struct_vtc_t vtc_t;



u32 vtc_is_locked(XVtc *inst);
u32 vtc_get_status(XVtc *inst);
u32 vtc_get_error(XVtc *inst);
void vtc_reset(XVtc *inst);
void vtc_get_timing(vtc_t *vtc);
float vtc_get_fps( vtc_t *vtc, float pclk_mhz );
void vtc_fill_timing(vtc_t *vtc);
void vtc_set_timing(vtc_t *vtc);
void vtc_print_timing(vtc_t *vtc);

//int vtc_init_common(u16 dev_id, vtc_t *vtc);
void vtc_init_gen(vtc_t *vtc);


#endif /* _VTC_HEADER_FILE_*/
